/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule;

import java.io.File;
import java.util.logging.Logger;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

/**
 * @author hunold
 *
 */
public class JeduleOptionParser {

    private final static Logger LOGGER = Logger.getLogger(JeduleOptionParser.class.getName());

    public JeduleOptionParser() {
    }

    public JeduleOptions parse(String[] args)
            throws JeduleCommandParseException {
        JeduleOptions jOptions = new JeduleOptions();

        Options options = new Options();
        options.addOption("f", "file", true, "jedule file");
        options.addOption("h", "help", false, "show this");
        options.addOption("p", "parser", true, "xml parser [simgrid,genmts]");
        options.addOption("o", "output", true, "output file");
        options.addOption("d", "dimension", true, "dimension of drawing");
        options.addOption("cm", "colormap", true, "color map");
        options.addOption("gt", "graphics type", true, "type of output file [jpg,png,pdf]");

        options.addOption("n", "normalize", false, "normalized time axis");

        CommandLineParser parser = new GnuParser();

        try {
            CommandLine line = parser.parse(options, args);

            if (line.hasOption("h")) {
                printHelp(options);
                jOptions.setValid(false);
            } else {

                if (line.hasOption("f")) {
                    jOptions.setJeduleFileName(line.getOptionValue("f"));
                } else {
                    jOptions.setValid(false);
                }

                if (line.hasOption("p")) {
                    jOptions.setJeduleFileType(line.getOptionValue("p"));
                } 

                if (line.hasOption("cm")) {
                    String cmOption = line.getOptionValue("cm");
                    if( new File(cmOption).exists() ) {
                        jOptions.setJeduleColorMapFile(cmOption);
                    } else {
                        jOptions.setJeduleColorMapName(cmOption);
                    }
                }

                if (line.hasOption("gt")) {
                    jOptions.setOutputType(line.getOptionValue("gt"));
                } else {
                    if( line.hasOption("o") ) {
                        LOGGER.warning("no file type specified: using default output type");
                    }
                }

                if (line.hasOption("o")) {
                    jOptions.setOutputFile(line.getOptionValue("o"));
                }

                if (line.hasOption("d")) {
                    final String dimString = line.getOptionValue("d");
                    final int xPos = dimString.indexOf('x');
                    final String errorMsg = "incorrect dimension\nignoring dimension...";

                    if (xPos == -1) {
                        System.err.println(errorMsg);
                    } else {
                        final String width = dimString.substring(0, xPos);
                        final String height = dimString.substring(xPos + 1,
                                dimString.length());
                        try {
                            int w = Integer.parseInt(width);
                            int h = Integer.parseInt(height);
                            jOptions.setWidth(w);
                            jOptions.setHeight(h);
                        } catch (NumberFormatException e) {
                            System.err.println(errorMsg);
                        }

                    }
                }

                if (line.hasOption("n")) {
                    jOptions.setNormalized(true);
                }

                if (!jOptions.isValid()) {
                    printHelp(options);
                }
            }

        } catch (ParseException e) {
            throw new JeduleCommandParseException(e);
        }

        return jOptions;
    }

    private void printHelp(Options options) {
        HelpFormatter formatter = new HelpFormatter();
        formatter.printHelp("JeduleStarter", options);
    }

}
