/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule;

import java.util.HashMap;
import java.util.logging.Logger;

import net.sf.jedule.settings.JeduleContextSupportedFormats.FILE_TYPE;

/**
 * @author hunold
 *
 */
public class JeduleOptions {

    private final static Logger LOGGER = Logger.getLogger(JeduleOptions.class.getName());

    private boolean valid = true;
    private String jeduleFileName;
    private String jeduleFileType = "simgrid";
    private String graphicsContext = "swing";
    private String outputFile;
    private int width = 1024;
    private int height = 768;
    private boolean normalized;
    private String jeduleColorMap = "default";
    private FILE_TYPE outputType = FILE_TYPE.PNG;
    private String jeduleColorMapFile;

    private static HashMap<String, FILE_TYPE> supportedTypes = new HashMap<String, FILE_TYPE>();
    static {
        supportedTypes.put("png", FILE_TYPE.PNG);
        supportedTypes.put("jpg", FILE_TYPE.JPG);
        supportedTypes.put("pdf", FILE_TYPE.PDF);
        //supportedTypes.put("tikz", FILE_TYPE.TIKZ);
    }

    public JeduleOptions() {

    }

    public boolean isValid() {
        return valid;
    }

    public void setValid(boolean valid) {
        this.valid = valid;
    }

    /**
     * @return the jeduleFileName
     */
    public String getJeduleFileName() {
        return jeduleFileName;
    }

    /**
     * @param jeduleFileName
     *            the jeduleFileName to set
     */
    public void setJeduleFileName(String jeduleFileName) {
        this.jeduleFileName = jeduleFileName;
    }

    /**
     * @return the jeduleFileType
     */
    public String getJeduleFileType() {
        return jeduleFileType;
    }

    /**
     * @param jeduleFileType
     *            the jeduleFileType to set
     */
    public void setJeduleFileType(String jeduleFileType) {
        this.jeduleFileType = jeduleFileType;
    }

    /**
     * @return the graphicsContext
     */
    public String getGraphicsContext() {
        return graphicsContext;
    }

    /**
     * @param graphicsContext
     *            the graphicsContext to set
     */
    public void setGraphicsContext(String graphicsContext) {
        this.graphicsContext = graphicsContext;
    }

    /**
     * @return the height
     */
    public int getHeight() {
        return height;
    }

    /**
     * @param height
     *            the height to set
     */
    public void setHeight(int height) {
        this.height = height;
    }

    /**
     * @return the outputFile
     */
    public String getOutputFile() {
        return outputFile;
    }

    /**
     * @param outputFile
     *            the outputFile to set
     */
    public void setOutputFile(String outputFile) {
        this.outputFile = outputFile;
    }

    /**
     * @return the width
     */
    public int getWidth() {
        return width;
    }

    /**
     * @param width the width to set
     */
    public void setWidth(int width) {
        this.width = width;
    }

    public boolean isNormalized() {
        return normalized;
    }

    public void setNormalized(boolean normalized) {
        this.normalized = normalized;
    }

    public void setJeduleColorMapName(String mapName) {
        this.jeduleColorMap = mapName;
    }

    public String getJeduleColorMapName() {
        return jeduleColorMap;
    }

    public void setOutputType(String type) {
        if( supportedTypes.containsKey(type) ) {
            outputType = supportedTypes.get(type);
        } else {
            LOGGER.warning("output type not supported : " + type
                    + " ... using default : " + getOutputType());
        }
    }

    public FILE_TYPE getOutputType() {
        return outputType;
    }

    public void setJeduleColorMapFile(String jeduleColorMapFile) {
        this.jeduleColorMapFile = jeduleColorMapFile;
    }

    public String getJeduleColorMapFile() {
        return jeduleColorMapFile;
    }


}
