/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import net.sf.jedule.parser.JeduleParserException;
import net.sf.jedule.ui.UnsupportedContextException;

/**
 * @author hunold
 *
 */
public class JeduleStarter {

    public static void main(String[] args) {

        initLogging();

        final JeduleOptionParser optParser = new JeduleOptionParser();

        JeduleOptions jOptions = null;
        try {
            jOptions = optParser.parse(args);
        } catch (JeduleCommandParseException e) {
            System.err.println(e.getMessage());
            e.printStackTrace();
            System.exit(-1);
        }

        if (jOptions != null && jOptions.isValid()) {

            try {

                new JeduleController(jOptions);

            } catch (JeduleParserException e) {
                System.err.println(e.getMessage());
                e.printStackTrace();
                System.exit(-1);
            } catch (UnsupportedContextException e) {
                System.err.println(e.getMessage());
                e.printStackTrace();
                System.exit(-1);
            }

        }

    }

    private static void initLogging() {
        final String logPropFileName = "jedule_logger.properties";

        try {
          LogManager.getLogManager().readConfiguration(
                    JeduleStarter.class.getResourceAsStream(logPropFileName));

        } catch (IOException e) {
          System.err.println("Cannot initialize logging. Error while reading "
              + logPropFileName);

          Logger.getLogger("").setLevel(Level.WARNING);
        }
    }

}
