/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule;

import java.util.List;

import net.sf.jedule.parser.JeduleParserException;
import net.sf.jedule.parser.JeduleXmlParser;
import net.sf.jedule.schedule.JeduleNode;
import net.sf.jedule.system.JeduleGrid;
import net.sf.jedule.system.JeduleMeta;

/**
 * @author hunold
 *
 */
public class JeduleXmlModel implements JeduleModel {

    private String jeduleFileName;
    private String jeduleType;

    private List<JeduleNode> nodeList;
    private JeduleGrid jeduleGrid;
    private JeduleMeta metaInfo;

    public JeduleXmlModel(final String jeduleFileName, final String jeduleType)
            throws JeduleParserException {
        this.jeduleFileName = jeduleFileName;
        this.jeduleType = jeduleType;
        loadFromFile();
    }

    private void loadFromFile() throws JeduleParserException {
        final JeduleXmlParser parser = JeduleParserFactory
                .getXmlParser(jeduleType);
        parser.setInputFileName(jeduleFileName);
        jeduleGrid = parser.readGrid();
        nodeList = parser.readNodes();
        metaInfo = parser.readMetaInfo();
        if (metaInfo == null) {
            metaInfo = new JeduleMeta();
        }
    }

    public JeduleGrid getJeduleGrid() {
        return jeduleGrid;
    }

    public List<JeduleNode> getNodeList() {
        return nodeList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see net.sf.jedule.JeduleModel#getMetaInfo()
     */
    public JeduleMeta getMetaInfo() {
        return metaInfo;
    }

    public void setJeduleGrid(JeduleGrid grid) {
        this.jeduleGrid = grid;
    }

    public void setNodeList(List<JeduleNode> nodeList) {
        this.nodeList = nodeList;
    }

    public void reload() throws JeduleParserException {
        loadFromFile();
    }

}
