/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.graphics;

import java.awt.Dimension;
import java.awt.Point;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import net.sf.jedule.ui.IJSheetGraphicsContextRenderable;

/**
 * @author hunold
 *
 */
public abstract class JSheetObject implements IJSheetGraphicsContextRenderable {

	private final HashMap<JSheetObject, RenderingInfo> childMap = new HashMap<JSheetObject, RenderingInfo>();

	private Dimension size = new Dimension(0, 0);
	private boolean visable = true;

	public JSheetObject(int width, int height) {
		this.size = new Dimension(width, height);
	}

	public Dimension getSize() {
		return size;
	}

	public void setSize(Dimension dimension) {
		this.size = dimension;
	}

	public void setSize(int width, int height) {
		this.size = new Dimension(width, height);
	}

	public int getHeight() {
		return getSize().height;
	}

	public int getWidth() {
		return getSize().width;
	}

	public void addChildObject(final JSheetObject child, final int x, final int y) {
		childMap.put(child, new RenderingInfo(x, y));
	}

	public void removeChildObject(final JSheetObject child) {
		childMap.remove(child);
	}
	
	public void removeAllChilds() {
	    childMap.clear();
	}

	public boolean hasChildObject(JSheetObject child) {
		return childMap.containsKey(child);
	}


	public List<JSheetObject> getChildObjects() {
		return new ArrayList<JSheetObject>(childMap.keySet());
	}

	public Point getChildPosition(JSheetObject child) {
		Point p = null;
		RenderingInfo info = childMap.get(child);
		if (info != null) {
			p = new Point(info.getX(), info.getY());
		}
		return p;
	}

	public void setVisable(boolean visable) {
        this.visable = visable;
    }

    public boolean isVisable() {
        return visable;
    }

    static class RenderingInfo {
		private final int x;
		private final int y;

		public RenderingInfo(final int x, final int y) {
			super();
			this.x = x;
			this.y = y;
		}

		public int getX() {
			return x;
		}

		public int getY() {
			return y;
		}
	}


}
