/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.graphics.composed;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Point2D;

import net.sf.jedule.graphics.GraphicsConstants;
import net.sf.jedule.graphics.JSheetObject;
import net.sf.jedule.graphics.GraphicsConstants.ZOOM_DIRECTION;
import net.sf.jedule.schedule.Jedule;

/**
 * @author sascha
 *
 */
public class JeduleGraphic extends JSheetObject {

    private final int clusterId;
    private JSheetRuler ruler;

    public JeduleGraphic(Jedule jedule, int clusterId, int width, int height) {
        super(width, height);
        this.clusterId = clusterId;
        ruler = new JSheetRuler(jedule, clusterId, width, height);
    }

    public int getClusterId() {
        return clusterId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see net.sf.jedule.ui.IJSheetGraphicsContextRenderable#doLayout()
     */
    public void doLayout() {
        ruler.setSize(getWidth(), getHeight());
        ruler.doLayout();
        addChildObject(ruler, 0, 0);
    }

    /**
     * @param rectangle
     */
    public void zoom(Rectangle rectangle) {
        ruler.zoom(rectangle);
        // ruler.doLayout();
    }

    /**
     * @param scaling
     *            scaling factor (0.1 <= s <= 1.0)
     * @param center
     *            point of zoom event in glasspane
     */
    public void zoom(float scaling, Point centerPoint) {
        ruler.zoom(scaling, centerPoint, ZOOM_DIRECTION.BOTH_AXES);
        // ruler.doLayout();
    }

    /**
     * @param scaling
     *            scaling factor (0.1 <= s <= 1.0)
     * @param center
     *            point of zoom event in glasspane
     */
    public void zoom(float scaling, Point centerPoint,
            GraphicsConstants.ZOOM_DIRECTION direction) {
        ruler.zoom(scaling, centerPoint, direction);
        // ruler.doLayout();
    }

    /**
     * @param point
     */
    public Point2D getCoordinates(Point point) {
        return ruler.getCoordinates(point);
    }

    /**
     * @param srcPoint
     * @param dstPoint
     */
    public void move(final Point srcPoint, final Point dstPoint) {
        Point2D rulerSrcPoint = ruler.getCoordinates(srcPoint);
        Point2D rulerDstPoint = ruler.getCoordinates(dstPoint);

        if (rulerSrcPoint == null || rulerDstPoint == null) {
            // points not inside ruler
            return;
        }

        final double xMove = rulerDstPoint.getX() - rulerSrcPoint.getX();
        final double yMove = rulerDstPoint.getY() - rulerSrcPoint.getY();

        final double xMin = ruler.getMinX() - xMove;
        final double xMax = ruler.getMaxX() - xMove;

        final double yMin = ruler.getMinY() - yMove;
        final double yMax = ruler.getMaxY() - yMove;

        ruler.setBoundsY(yMin, yMax);
        ruler.setBoundsX(xMin, xMax);
    }

    /**
     * call this to zoom out
     * 
     */
    public void resetView() {
        ruler.initView();
        ruler.doLayout();
    }

    /**
     * @param min
     * @param max
     */
    public void setBoundsY(double min, double max) {
        ruler.setGlobalBoundsY(min, max);
    }

    /**
     * @return
     */
    public double getMinY() {
        return ruler.getMinY();
    }

    /**
     * @return
     */
    public double getMaxY() {
        return ruler.getMaxY();
    }

}
