/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.schedule;

import java.util.List;

import net.sf.jedule.JeduleModel;
import net.sf.jedule.parser.JeduleParserException;
import net.sf.jedule.system.JeduleGrid;
import net.sf.jedule.system.JeduleMeta;

/**
 * @author hunold
 *
 */
public class Jedule {

    private static int jeduleId = 0;
    private final JeduleModel model;
    private final String name;

    public Jedule(JeduleModel model) {
        this(model, null);
    }

    public Jedule(JeduleModel model, String name) {
        this.model = model;

        if (name == null) {
            this.name = makeJeduleName();
        } else {
            this.name = name;
        }
    }

    private String makeJeduleName() {
        return "Jedule" + jeduleId++;
    }

    public String getName() {
        return name;
    }

    public void reload() throws JeduleParserException {
        model.reload();
    }

    public JeduleMeta getMetaInfo() {
        return model.getMetaInfo();
    }

    // public JeduleModel getModel() {
    // return model;
    // }

    public JeduleGrid getJeduleGrid() {
        return model.getJeduleGrid();
    }

    public List<JeduleNode> getJeduleNodeList() {
        return model.getNodeList();
    }

    public double getFinishTime(int clusterId) {
        double finishTime = -1.0;
        for (JeduleNode node : getJeduleNodeList()) {
            if (node.getClusterId() == clusterId) {
                if (node.getEndTime() > finishTime) {
                    finishTime = node.getEndTime();
                }
            }
        }
        return finishTime;
    }

    /**
     * @param nodeId
     * @return
     */
    public JeduleNode getJeduleNode(String nodeId) {
        JeduleNode node = null;
        for (JeduleNode n : getJeduleNodeList()) {
            if (n.getId().equals(nodeId)) {
                node = n;
                break;
            }
        }
        return node;
    }

}
