/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.schedule;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import net.sf.jedule.system.JeduleHost;

/**
 * @author hunold
 *
 */
public class JeduleNode {

//    private final static Logger LOGGER = Logger.getLogger(JeduleNode.class
//            .getName());

	private final String id;
	private final String nodeType;
	private final double startTime;
	private final double endTime;
	private final int clusterId;
	private final List<JeduleHost> hostList;

	public JeduleNode(final String id, final double startTime,
			final double endTime, final int clusterId, final List<JeduleHost> hostList,
			final String type) {
		this.id = id;
		this.startTime = startTime;
		this.endTime = endTime;
		this.clusterId = clusterId;
		this.hostList = hostList;
		this.nodeType = type;

//		LOGGER.info("node : " + id);
//		StringBuffer buf = new StringBuffer();
//		for(int i=0; i<hostList.size(); i++) {
//		    buf.append(hostList.get(i).getHostId() + " ");
//		}
//		LOGGER.info("hosts: " + buf);
	}

	/**
   * @return the id
   */
	public String getId() {
		return this.id;
	}

	public int getClusterId() {
		return this.clusterId;
	}

	public List<String> getHostIdList() {
		final List<String> hostIdList = new ArrayList<String>();
		for(JeduleHost h : hostList) {
			hostIdList.add( Integer.toString( h.getHostId() ) );
		}
		return hostIdList;
	}

	public List<JeduleTask> getJeduleTaskList() {
		List<JeduleTask> taskList = new ArrayList<JeduleTask>();
		List<JeduleHost> hostList2 = new ArrayList<JeduleHost>(hostList);
		Collections.sort(hostList2, new JeduleHostComparator());

		JeduleTask task = null;
		for(JeduleHost h : hostList2) {
			if( task == null ) {
				task = new JeduleTask(id, h.getHostId(), h.getHostId(), startTime,
						endTime, getNodeType());
			}
			if( h.getHostId() > task.getEndHost() + 1 ) {
//	            LOGGER.info("add task: " + id + " start: " + task.getStartHost()
//	                    + " .." + task.getEndHost());
				taskList.add(task);
				task = new JeduleTask(id, h.getHostId(), h.getHostId(), startTime,
						endTime, getNodeType());
			} else {
				task.setEndHost(h.getHostId());
			}
		}
		if( task != null && !taskList.contains(task) ) {
//		    LOGGER.info("add task: " + id + " start: " + task.getStartHost()
//                    + " .." + task.getEndHost());
			taskList.add(task);
		}

		return taskList;
	}

	static class JeduleHostComparator implements Comparator<JeduleHost>, Serializable {
		public int compare(final JeduleHost h1, final JeduleHost h2) {
			return (h1.getHostId() - h2.getHostId());
		}
	}

	/**
	 * @return the endTime
	 */
	public double getEndTime() {
		return endTime;
	}

	/**
	 * @return the startTime
	 */
	public double getStartTime() {
		return startTime;
	}

	/**
	 * @return the nodeType
	 */
	public String getNodeType() {
		return nodeType;
	}


	public void addHost(final JeduleHost host) {
		if( ! hostList.contains(host) ) {
			hostList.add(host);
		}
	}

}
