/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.schedule;

import java.io.PrintStream;

public class JeduleTask {

	private final String nodeId;
	private final String nodeType;
	private int startHost;
	private int endHost;
	private double startTime;
	private double endTime;

	public JeduleTask(final String nodeId, final int startHost,
			final int endHost, double startTime, double endTime, String nodeType) {
		super();
		this.nodeId = nodeId;
		this.startHost = startHost;
		this.endHost = endHost;
		this.startTime = startTime;
		this.endTime = endTime;
		this.nodeType = nodeType;
	}

	public JeduleTask(JeduleTask t) {
		this(t.getNodeId(), t.getStartHost(), t.getEndHost(), t.getStartTime(), t
				.getEndTime(), t.getNodeType());
	}

	public int getEndHost() {
		return endHost;
	}

	public String getNodeId() {
		return nodeId;
	}

	public int getStartHost() {
		return startHost;
	}

	public void setEndHost(int endHost) {
		this.endHost = endHost;
	}

	public double getEndTime() {
		return endTime;
	}

	public double getStartTime() {
		return startTime;
	}

	public String getNodeType() {
		return nodeType;
	}

	/**
	 * @param endTime the endTime to set
	 */
	public void setEndTime(double endTime) {
		this.endTime = endTime;
	}

	/**
	 * @param startHost the startHost to set
	 */
	public void setStartHost(int startHost) {
		this.startHost = startHost;
	}

	/**
	 * @param startTime the startTime to set
	 */
	public void setStartTime(double startTime) {
		this.startTime = startTime;
	}

	public void print(PrintStream out) {
        out.println(toString());
    }

    public String toString() {
        return "ID " + getNodeId() + " time [ " + getStartTime() + " , "
                + getEndTime() + " ] , hosts [ " + getStartHost() + " , "
                + getEndHost() + "]";
    }

	@Override
	public boolean equals(Object obj) {
		boolean equal = false;
				
		if( obj instanceof JeduleTask ) {
			JeduleTask other = (JeduleTask)obj;
			
			if( other.getNodeId().compareTo(getNodeId()) != 0 ) {
				equal = false;
			} else if( other.getStartHost() != getStartHost() ) {
				equal = false;
			} else if( other.getEndHost() != getEndHost() ) {
				equal = false;
			} else if( other.getStartTime() != getStartTime() ) {
				equal = false;
			} else if( other.getEndTime() != getEndTime() ) {
				equal = false;
			} else if( other.getNodeType() != getNodeType() ) {
				equal = false;
			} else {
				System.out.println("------------------");
				equal = true;
			}
		}
		
		return equal;
	}

    
    
}
