/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.schedule;

import java.util.Comparator;

/**
 * @author hunold
 *
 */
public class JeduleTaskComparator implements Comparator<JeduleTask> {

	public enum COMPARE_TYPE { START_TIME, FIRST_PROCESSOR };

	private final COMPARE_TYPE type;

	public JeduleTaskComparator(COMPARE_TYPE type) {
		this.type = type;
	}

	/* (non-Javadoc)
	 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	 */
	public int compare(JeduleTask o1, JeduleTask o2) {
		int result = 0;

		switch (type) {
		case START_TIME:
			if (o1.getStartTime() < o2.getStartTime()) {
				result = -1;
			} else if (o1.getStartTime() == o2.getStartTime()) {
				result = 0;
			} else {
				result = 1;
			}
			break;
		case FIRST_PROCESSOR:
			result = (o1.getStartHost() - o2.getStartHost());
			break;
		default:
			System.err.println("unknown compare type " + type);
			break;
		}
		return result;
	}

}
