/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.schedule;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.sf.jedule.schedule.JeduleTaskComparator.COMPARE_TYPE;

/**
 * @author hunold
 *
 */
public class JeduleToolkit {

//	private final static Logger LOGGER = Logger.getLogger(JeduleToolkit.class
//			.getName());


	/**
   *
   * @param taskList
   * @return returns a list of task lists, each task list contains overlapping
   *         tasks,in particular, 2 different lists of tasks do not overlap
   */

	public static List<List<JeduleTask>> getOverlappingTasks(
			final List<JeduleTask> taskList) {

		// scan by time axis
		final List<JeduleTask> list1 = sortTasks(taskList, COMPARE_TYPE.START_TIME);
		final List<List<JeduleTask>> overlapList = new ArrayList<List<JeduleTask>>();
		List<JeduleTask> currentList = new ArrayList<JeduleTask>();
		for(JeduleTask task : list1) {
			//System.out.println("task " + task.getNodeId() + " start time: " + task.getStartTime());
			if( currentList.size() > 0 ) {

				double currentMinTime = currentList.get(0).getStartTime();
				double currentMaxTime = currentList.get(0).getEndTime();

				for(final JeduleTask curTask : currentList) {
					if( curTask.getStartTime() < currentMinTime ) {
						currentMinTime = curTask.getStartTime();
					}
					if( curTask.getEndTime() > currentMaxTime ) {
						currentMaxTime = curTask.getEndTime();
					}
				}

				if( task.getStartTime() >= currentMaxTime ) {
					overlapList.add(currentList);
					currentList = new ArrayList<JeduleTask>();
					currentList.add(task);
				} else {
					currentList.add(task);
				}
			} else {
				currentList.add(task);
			}
		}
		if( currentList.size() > 0 ) {
			overlapList.add(currentList);
		}

//		for(List<JeduleTask> ovList : overlapList) {
//			System.out.print("time list : ");
//			for(final JeduleTask jT : ovList) {
//				System.out.print(jT.getNodeId() + " ");
//			}
//			System.out.println();
//		}

		// scan by processors
		final List<List<JeduleTask>> returnList = new ArrayList<List<JeduleTask>>();
		for( List<JeduleTask> timeList : overlapList ) {
			final List<JeduleTask> list2 = sortTasks(timeList, COMPARE_TYPE.FIRST_PROCESSOR);

			List<JeduleTask> procList = new ArrayList<JeduleTask>();
			JeduleTask lastProcTask = null;
			for(JeduleTask timeTask : list2 ) {
				if( procList.size() == 0 ) {
					procList.add(timeTask);
					lastProcTask = timeTask;
				} else {
					if( lastProcTask.getEndHost() < timeTask.getStartHost() ) {
						returnList.add(procList);
						procList = new ArrayList<JeduleTask>();
						procList.add(timeTask);
						lastProcTask = timeTask;
					} else {
						procList.add(timeTask);
						if( lastProcTask.getEndHost() < timeTask.getEndHost() ) {
							lastProcTask = timeTask;
						}
					}
				}
			}
			if( procList.size() > 0 ) {
				returnList.add(procList);
			}
		}

//        for (List<JeduleTask> retList : returnList) {
//            System.out.print("proc list : ");
//            for (final JeduleTask jT : retList) {
//                System.out.print(jT.getNodeId() + " " + "[" + jT.getStartHost()
//                        + "," + jT.getEndHost() + "] ");
//            }
//            System.out.println();
//        }


		return returnList;
	}

	/**
	 * @param taskList
	 * @param type
	 * @return
	 */
	private static List<JeduleTask> sortTasks(List<JeduleTask> taskList,
			COMPARE_TYPE type) {
		final List<JeduleTask> copyList = new ArrayList<JeduleTask>(taskList);
		Collections.sort(copyList, new JeduleTaskComparator(type));
		return copyList;
	}

	/**
	 * each list is logical rectangle which is shared by the tasks in this list
	 *
	 * @param list
	 * @return
	 */
	public static List<List<JeduleTask>> partitionTasks(List<JeduleTask> list) {

		final List<List<JeduleTask>> partitionedList = new ArrayList<List<JeduleTask>>();

		final List<Integer> procBounds = getAllXTics(list);
		Collections.sort(procBounds);

//        for (Integer procBound : procBounds) {
//            System.out.print(procBound + " ");
//        }
//        System.out.println();

		final List<List<JeduleTask>> procPartList = partitionByProcessors(list,
		        procBounds);

//        for (List<JeduleTask> l : procPartList) {
//        	System.out.println("-- partitionedByProcessors -- ");
//            for( JeduleTask t : l) {
//                System.out.println(t);
//            }
//        }
        

		// scan by y-axis
		final List<Double> timeBounds = getAllYTics(list);
		Collections.sort(timeBounds);

		for (int i = 0; i < timeBounds.size() - 1; i++) {
			final double bottomBound = timeBounds.get(i);
			final double topBound = timeBounds.get(i + 1);

			for (List<JeduleTask> partList : procPartList) {
				//System.out.println("********************");
				final List<JeduleTask> boundList = new ArrayList<JeduleTask>();
				for (JeduleTask t : partList) {
					if (bottomBound >= t.getStartTime() && topBound <= t.getEndTime()) {
						JeduleTask miniTask = new JeduleTask(t.getNodeId(), t
								.getStartHost(), t.getEndHost(), bottomBound, topBound, t
								.getNodeType());
						//System.out.println("create minitask : " + miniTask);
						if( ! boundList.contains(miniTask) ) {
							//System.out.println("add created minitask : " + miniTask);
							boundList.add(miniTask);
						}
					}
				}
				if (boundList.size() > 0) {
					partitionedList.add(boundList);
				}
			}
		}

		// printTaskLists(partitionedList);

		//  merge in x direction
		mergeListsByProcessors(partitionedList);

		// merge in y direction
		mergeListsByTime(partitionedList);

		//	printTaskLists(partitionedList);

		return partitionedList;
	}


//	private static void printTaskLists(List<List<JeduleTask>> taskLists) {
//		int c = 0;
//		for(List<JeduleTask> l : taskLists) {
//			System.out.println("---- proc part list ----" + (c++));
//			for(JeduleTask t : l) {
//				t.print(System.out);
//			}
//		}
//	}


	/**
     * this functions iterates over processor bounds and checks for each task if
     * it is part of the currently considered interval, if so, a minitask is
     * created with the id of the task and the current interval
     *
     *
     * @param list
     * @param procBounds
     * @param procBoundsEnd
     * @return
     */
    private static List<List<JeduleTask>> partitionByProcessors(
            List<JeduleTask> list, List<Integer> procBounds) {

        final List<List<JeduleTask>> procPartList = new ArrayList<List<JeduleTask>>();


        if( procBounds.size() > 0 ) {
            int max = procBounds.get(procBounds.size()-1);

            for(int p=0; p<=max; p++) {
                int leftBound  = p;
                int rightBound = p;

//                System.out.println("left proc " + leftBound + " right proc "
//                        + rightBound);
                final List<JeduleTask> boundList = new ArrayList<JeduleTask>();
                for (final JeduleTask t : list) {
//                    System.out.println("check task: " + t);

                    boolean addMini = false;
                    int miniLB = 0;
                    int miniRB = 0;

                    // case 1 : lb sh rb eh
                    if (leftBound <= t.getStartHost()
                            && t.getStartHost() <= rightBound
                            && rightBound <= t.getEndHost()) {
                        addMini = true;
                        miniLB = t.getStartHost();
                        miniRB = rightBound;
                        // case 2: lb sh eh rb
                    } else if (leftBound <= t.getStartHost()
                            && t.getStartHost() <= rightBound
                            && t.getEndHost() <= rightBound) {
                        addMini = true;
                        miniLB = t.getStartHost();
                        miniRB = t.getEndHost();
                        // case 3: sh lb eh rb
                    } else if (t.getStartHost() <= leftBound
                            && leftBound <= t.getEndHost()
                            && t.getEndHost() <= rightBound) {
                        addMini = true;
                        miniLB = leftBound;
                        miniRB = t.getEndHost();
                        // case 4: sh lb rb rh
                    } else if (t.getStartHost() <= leftBound
                            && leftBound <= rightBound
                            && rightBound <= t.getEndHost()) {
                        addMini = true;
                        miniLB = leftBound;
                        miniRB = rightBound;
                    }

                    if (addMini) {
                        final JeduleTask miniTask = new JeduleTask(t.getNodeId(),
                                miniLB, miniRB, t.getStartTime(), t.getEndTime(), t
                                        .getNodeType());
//                        System.out.println("add minitask : " + miniTask);
                        boundList.add(miniTask);
                    }

                }

                if (boundList.size() > 0) {
                    // System.out.println("ADD minitasks>>>>>>>>>>>");
                    // for(JeduleTask m : boundList) {
                    // System.out.println("WILL add minitask : " + m);
                    // }

                    procPartList.add(boundList);
                }


            }

        }




        return procPartList;
    }

	/**
     * @param list
     */
	private static List<Double> getAllYTics(List<JeduleTask> list) {
		final List<Double> ticList = new ArrayList<Double>();
		for(JeduleTask t : list) {
			if( ! ticList.contains(t.getStartTime())) {
				ticList.add(t.getStartTime());
			}
			if( ! ticList.contains(t.getEndTime())) {
				ticList.add(t.getEndTime());
			}
		}
		return ticList;
	}

	/**
	 * @param list
	 * @return
	 */
	private static List<Integer> getAllXTics(List<JeduleTask> list) {
		List<Integer> ticList = new ArrayList<Integer>();
		for(JeduleTask t : list) {
			if( ! ticList.contains(t.getStartHost())) {
				ticList.add(t.getStartHost());
			}
            if( ! ticList.contains(t.getEndHost())) {
                ticList.add(t.getEndHost());
            }
        }
        return ticList;
    }

	/**
	 * @param procPartList
	 *
	 */
	private static void mergeListsByTime(List<List<JeduleTask>> taskLists) {

		// try to merge lists
		// possible if endP proc of list1 = (start proc - 1) of list2
		// AND if lists contain same elements

		for (int i = 0; i < taskLists.size(); i++) {

			final List<JeduleTask> toExpand = taskLists.get(i);

			for (int j = i + 1; j < taskLists.size();) {
				final List<JeduleTask> list2 = taskLists.get(j);

				if (haveAdjacentTimeBounds(toExpand, list2)
						&& haveSameTaskIds(toExpand, list2)
						&& haveSameProcessorRange(toExpand, list2)) {
					final double newStartTime = Math.min(toExpand.get(0).getStartTime(),
							list2.get(0).getStartTime());
					final double newEndTime = Math.max(toExpand.get(0).getEndTime(),
							list2.get(0).getEndTime());

					for (final JeduleTask t : toExpand) {
						t.setStartTime(newStartTime);
						t.setEndTime(newEndTime);
					}

					taskLists.remove(list2);
				} else {
					j++;
				}
			}
		}
	}

	/**
	 * @param list1
	 * @param list2
	 * @return
	 */
	private static boolean haveSameProcessorRange(List<JeduleTask> list1,
			List<JeduleTask> list2) {
		boolean sameRange = false;

		if (list1.size() > 0 && list2.size() > 0) {
			JeduleTask t1 = list1.get(0);
			JeduleTask t2 = list2.get(0);
			if (t1.getStartHost() == t2.getStartHost()
					&& t1.getEndHost() == t2.getEndHost()) {
				sameRange = true;
			}
		}

		return sameRange;
	}

	/**
	 * @param taskLists
	 *
	 */
	private static void mergeListsByProcessors(List<List<JeduleTask>> taskLists) {

		// try to merge lists
		// possible if endP proc of list1 = (start proc - 1) of list2
		// AND if lists contain same elements

		for (int i = 0; i < taskLists.size(); i++) {

			final List<JeduleTask> toExpand = taskLists.get(i);

			for (int j = i + 1; j < taskLists.size();) {
				final List<JeduleTask> list2 = taskLists.get(j);

				if (haveSameTaskIds(toExpand, list2)
						&& haveAdjacentHostBounds(toExpand, list2)
						&& haveSameTimeRange(toExpand, list2)) {

					final int newProcStart = Math.min(toExpand.get(0).getStartHost(),
							list2.get(0).getStartHost());
					final int newProcEnd = Math.max(toExpand.get(0).getEndHost(), list2
							.get(0).getEndHost());

					for (final JeduleTask t : toExpand) {
						t.setStartHost(newProcStart);
						t.setEndHost(newProcEnd);
						// System.out.println("create new merged task");
						// mergedTask.print(System.out);
					}

					taskLists.remove(list2);
				} else {
					j++;
				}
			}
		}
	}

	/**
	 * @param list1
	 * @param list2
	 * @return
	 */
	private static boolean haveSameTimeRange(List<JeduleTask> list1,
			List<JeduleTask> list2) {
		boolean sameRange = false;

		if (list1.size() > 0 && list2.size() > 0) {
			JeduleTask t1 = list1.get(0);
			JeduleTask t2 = list2.get(0);
			if (t1.getStartTime() == t2.getStartTime()
					&& t1.getEndTime() == t2.getEndTime()) {
				sameRange = true;
			}
		}

		return sameRange;
	}

	/**
   * @param list1
   * @param list2
   * @return
   */
	private static boolean haveSameTaskIds(List<JeduleTask> list1,
			List<JeduleTask> list2) {
		boolean sameIds = true;

		if (list1.size() == list2.size()) {
			for (int i = 0; i < list1.size(); i++) {
				if (!list1.get(i).getNodeId().equals(list2.get(i).getNodeId())) {
					sameIds = false;
					break;
				}
			}
		} else {
			sameIds = false;
		}

		return sameIds;
	}

	/**
	 * @param list1
	 * @param list2
	 * @return
	 */
	private static boolean haveAdjacentHostBounds(List<JeduleTask> list1,
			List<JeduleTask> list2) {
		boolean adjacent = false;

		if (list1.size() > 0 && list2.size() > 0) {
			final JeduleTask t1 = list1.get(0);
			final JeduleTask t2 = list2.get(0);
			if (t1.getEndHost() == t2.getStartHost() - 1
					|| t2.getEndHost() == t1.getStartHost() - 1) {
				adjacent = true;
			}
		}
		return adjacent;
	}

	/**
	 * @param list1
	 * @param list2
	 * @return
	 */
	private static boolean haveAdjacentTimeBounds(List<JeduleTask> list1,
			List<JeduleTask> list2) {
		boolean adjacent = false;
		if (list1.size() > 0 && list2.size() > 0) {
			final JeduleTask t1 = list1.get(0);
			final JeduleTask t2 = list2.get(0);
			if (t1.getEndTime() == t2.getStartTime()
					|| t2.getEndTime() == t1.getStartTime()) {
				adjacent = true;
			}
		}
		return adjacent;
	}

}
