/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.settings.global;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Logger;

/**
 * @author sascha
 *
 */
public class JeduleColorConfig {

    private final static Logger LOGGER = Logger
            .getLogger(JeduleColorConfig.class.getName());

    private final HashMap<String, JeduleColorMap> colorMaps = new HashMap<String, JeduleColorMap>();

    private JeduleColorMap activeMap;

    public JeduleColorConfig() {

    }

    public void addJeduleColorMap(JeduleColorMap colorMap) {
        colorMaps.put(colorMap.getName(), colorMap);
        if( activeMap == null ) {
            setActiveJeduleColorMap(colorMap.getName());
        }
    }

    public JeduleColorMap getJeduleColorMap(String colorMapName) {
        return colorMaps.get(colorMapName);
    }

    public void setActiveJeduleColorMap(String colorMapName) {
        if( colorMaps.containsKey(colorMapName)) {
            activeMap = colorMaps.get(colorMapName);
        } else {
            LOGGER.warning("no color map with name \"" + colorMapName
                    + "\"! ... using previous map");
        }
    }

    public JeduleColorMap getActiveJeduleColorMap() {
        return activeMap;
    }

    public List<String> getJeduleColorMapNames() {
        return new ArrayList<String>(colorMaps.keySet());
    }
    
   

}
