/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.settings.global;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

/**
 * @author sascha
 *
 */
public class JeduleColorMap {

    private final static Logger LOGGER = Logger.getLogger(JeduleColorMap.class.getName());

    private final String name;

    private Map<String, HashMap<String, Color>> taskToColorMap = new HashMap<String, HashMap<String,Color>>();
    
    private int minFontSizeLabels = 11;
    private int fontSizeLabels = 13;
    private int fontSizeAxes   = 12;
    

    public JeduleColorMap(final String name) {
        if (name == null) {
            throw new IllegalArgumentException("name of cmap must not be null");
        }
        this.name = name;
    }

    public void addColor(final String taskId, String colorId, final Color color) {
        HashMap<String, Color> colorMap = null;
        if( taskToColorMap.containsKey(taskId) ) {
            colorMap = taskToColorMap.get(taskId);
        } else {
            colorMap = new HashMap<String, Color>();
            taskToColorMap.put(taskId, colorMap);
        }
        colorMap.put(colorId, color);
    }

    public Color getColor(final String taskId, final String colorId) {
        Color c = null;
        if( taskToColorMap.containsKey(taskId) ) {
            final HashMap<String, Color> colorMap = taskToColorMap.get(taskId);
            if( colorMap.containsKey(colorId) ) {
                c = colorMap.get(colorId);
            }
        }
        if (c == null) {
            LOGGER.warning("color undefined for task " + taskId + " and ID "
                    + colorId);
            if( colorId.equals("fg") ) {
                c = Color.BLACK;
            } else {
                c = Color.WHITE;
            }
        }
        return c;
    }

    public String getName() {
        return name;
    }

    public int getFontSizeLabels() {
        return fontSizeLabels;
    }

    public void setFontSizeLabels(final int fontSizeLabels) {
        this.fontSizeLabels = fontSizeLabels;
    }

    public int getFontSizeAxes() {
        return fontSizeAxes;
    }

    public void setFontSizeAxes(final int fontSizeAxes) {
        this.fontSizeAxes = fontSizeAxes;
    }

    public int getMinFontSizeLabels() {
        return minFontSizeLabels;
    }

    public void setMinFontSizeLabels(int minFontSizeLabels) {
        this.minFontSizeLabels = minFontSizeLabels;
    }
    
    

}
