/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.ui;

import java.io.IOException;
import java.util.List;

import net.sf.jedule.schedule.Jedule;
import net.sf.jedule.settings.JeduleContextSupportedFormats;
import net.sf.jedule.settings.JeduleSaveInformation;
import net.sf.jedule.ui.event.IJSheetContextEventObject;

/**
 * @author hunold
 *
 */
public interface IJSheetGraphicsContext extends IJSheetContextEventObject {

    public void setJedule(Jedule jedule);

    public Jedule getJedule();

    public void show(int width, int height);

    public void save(final JeduleSaveInformation saveSpec) throws IOException;

    public void makeSnapshot() throws IOException;

    public void normalize();

    public void update();

    public void close();

    public void showNodeInformation(List<String> nodeIdList);

    public void enableCluster(int clusterId, boolean enabled);

    public void showClusters(List<Integer> clusterIdList);

    /**
     * a context has to provide a way to enter a file name of a jedule e.g. in
     * Swing, the user is prompted for a file name using the file chooser dialog
     *
     * @return returns the file name
     */
    public String getFileName();

    public JeduleSaveInformation getJeduleSaveInformation();

    public void markNodes(List<String> nodeIdList);

    public IJSheetErrorReporter getJSheetErrorReporter();

    public JeduleContextSupportedFormats getJeduleContextSupportedFormats();

    /**
     * @param message
     */
    public void showErrorDialog(String message);

}
