/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.ui.swing;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.logging.Logger;

import net.sf.jedule.graphics.JSheetLabeledRectangle;
import net.sf.jedule.graphics.JSheetRectangle;
import net.sf.jedule.graphics.composed.JSheetRuler;
import net.sf.jedule.graphics.composed.JSheetTaskRectangle;
import net.sf.jedule.ui.IJSheetGraphicsContextComponent;
import net.sf.jedule.ui.IJSheetGraphicsContextRenderable;
import net.sf.jedule.ui.swing.components.JSheetSwingComponent;
import net.sf.jedule.ui.swing.components.JSheetSwingLabeledRectangle;
import net.sf.jedule.ui.swing.components.JSheetSwingRectangle;
import net.sf.jedule.ui.swing.components.JSheetSwingRuler;
import net.sf.jedule.ui.swing.components.JSheetSwingTaskRectangle;

/**
 * @author hunold
 *
 */
public class JSheetSwingComponentFactory {

    private final HashMap<Class, Class> classMap = new HashMap<Class, Class>();
    private final static Logger LOGGER = Logger
            .getLogger(JSheetSwingComponentFactory.class.getName());

    public JSheetSwingComponentFactory() {
        registerClasses();
    }

    private void registerClasses() {
        classMap.put(JSheetRuler.class, JSheetSwingRuler.class);
        classMap.put(JSheetTaskRectangle.class, JSheetSwingTaskRectangle.class);
        classMap.put(JSheetLabeledRectangle.class,
                JSheetSwingLabeledRectangle.class);
        classMap.put(JSheetRectangle.class, JSheetSwingRectangle.class);
    }

    public IJSheetGraphicsContextComponent getIJSheetGraphicsContextComponent(
            IJSheetGraphicsContextRenderable renderable) {

        IJSheetGraphicsContextComponent component = null;

        if (classMap.containsKey(renderable.getClass())) {
            Class toInst = classMap.get(renderable.getClass());
            try {
                Constructor co = toInst.getConstructor(renderable.getClass());
                component = (JSheetSwingComponent) co.newInstance(renderable);
            } catch (NoSuchMethodException e) {
                LOGGER.severe(e.getMessage());
                e.printStackTrace();
            } catch (InstantiationException e) {
                LOGGER.severe(e.getMessage());
                e.printStackTrace();
            } catch (IllegalArgumentException e) {
                LOGGER.severe(e.getMessage());
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                LOGGER.severe(e.getMessage());
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                LOGGER.severe(e.getMessage());
                e.printStackTrace();
            }

        } else {
            LOGGER.warning("known renderable type " + renderable);
        }

        return component;
    }

}
