/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.ui.swing;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;

/**
 * @author hunold
 *
 */
public class JeduleClusterSelection implements ActionListener {


	private int clusterNum;

	private JButton okButton = new JButton("OK");
	private JButton cancelButton = new JButton("Cancel");
	private JDialog dialog;
	private boolean pressedOk;
	private JList selectionList = new JList();

	public JeduleClusterSelection(int clusterNum) {
		this.clusterNum = clusterNum;

		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
	}

	/**
	 * @param bar
	 * @return
	 */
	public boolean select(JFrame parent) {

		final Object[] listModel = new Object[clusterNum];
		for(int i=0; i<clusterNum; i++) {
			listModel[i] = "Cluster " + Integer.toString(i);
		}
		selectionList.setListData(listModel);

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);

		pressedOk = false;
		dialog = new JDialog(parent, "Cluster selection", true);

		dialog.getContentPane().add(selectionList, BorderLayout.CENTER);
		dialog.getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		dialog.pack();
		dialog.setLocationRelativeTo(dialog.getOwner());

		dialog.setVisible(true);

		return pressedOk;
	}

	/**
	 * @return list of selected cluster ids
	 */
	public List<Integer> getSelectedIds() {
		final List<Integer> idList = new ArrayList<Integer>();
		for(Integer id : selectionList.getSelectedIndices()) {
			idList.add(id);
		}
		return idList;
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		if( cmd.equals("ok") ) {
			pressedOk = true;
			dialog.setVisible(false);
		} else if ( cmd.equals("cancel") ) {
			dialog.setVisible(false);
		}
	}

}
