/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.ui.swing;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.jedule.schedule.JeduleNode;

/**
 * @author hunold
 *
 */
public class SwingJeduleNodeInfoViewer extends JFrame implements WindowFocusListener {

	private final List<JeduleNode> nodeList;

	public SwingJeduleNodeInfoViewer(List<JeduleNode> nodeList) {
		super("NodeInfo");
		this.nodeList = nodeList;
		initUI();
		addWindowFocusListener(this);
		pack();
	}

	private void initUI() {
		if( nodeList == null ) {
			return;
		}

		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

		for(JeduleNode n : nodeList) {
			mainPanel.add(createNodePanel(n));
		}

		getContentPane().add(mainPanel, BorderLayout.CENTER);

		setUndecorated(true);
	}


	protected JPanel createNodePanel(final JeduleNode node) {
		final JPanel p = new JPanel();
		final BoxLayout boxLayout = new BoxLayout(p, BoxLayout.Y_AXIS);
		p.setLayout(boxLayout);

		p.add( getKeyValuePanel("start time", Double.toString(node.getStartTime())));
		p.add( getKeyValuePanel("finish time", Double.toString(node.getEndTime())));

		p.setBorder(BorderFactory.createTitledBorder("ID: " + node.getId()));
		return p;
	}

	protected JPanel getKeyValuePanel(final String key, final String value) {
		final JPanel p = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
		final JTextField keyLabel = new JTextField(8);
		final JTextField valueLabel = new JTextField();

		keyLabel.setText(key);
		keyLabel.setEditable(false);
		keyLabel.setBorder(null);
		valueLabel.setText(value);
		valueLabel.setEditable(false);
		valueLabel.setBorder(null);
		p.add(keyLabel);
		p.add(valueLabel);
		return p;
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowFocusListener#windowGainedFocus(java.awt.event.WindowEvent)
	 */
	public void windowGainedFocus(WindowEvent e) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowFocusListener#windowLostFocus(java.awt.event.WindowEvent)
	 */
	public void windowLostFocus(WindowEvent e) {
		setVisible(false);
		dispose();
	}

}
