/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.ui.swing.components;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;

import net.sf.jedule.graphics.JSheetLabeledRectangle;
import net.sf.jedule.settings.global.JeduleColorMap;
import net.sf.jedule.settings.global.JeduleSettings;
import net.sf.jedule.ui.tools.ScreenTools;

public class JSheetSwingLabeledRectangle extends JSheetSwingRectangle {

    private static final long serialVersionUID = 1L;

    private Font labelFont;
    private int minFontSize = 10;

    public JSheetSwingLabeledRectangle(JSheetLabeledRectangle rectangle) {
        super(rectangle);
        final JeduleColorMap colorMap = JeduleSettings.getInstance()
                .getJeduleColorConfig().getActiveJeduleColorMap();
        final int fontSize = colorMap.getFontSizeLabels();
        labelFont = new Font("Monospaced", Font.PLAIN, fontSize);
        minFontSize = colorMap.getMinFontSizeLabels();
    }

    public Font getLabelFont() {
        return labelFont;
    }

    // public void setLabelFont(Font labelFont) {
    // this.labelFont = labelFont;
    // }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.sf.jedule.ui.swing.components.JSheetSwingRectangle#renderComponent
     * (java.awt.Graphics2D)
     */
    @Override
    public void renderComponent(Graphics2D g) {
        super.renderComponent(g);
        JSheetLabeledRectangle rect = (JSheetLabeledRectangle) getJSheetObject();
        g.setColor(getForeground());
        g.setFont(getLabelFont());

        if( getHeight() >= minFontSize) {
            String label = rect.getLabel();
            if (ScreenTools.getStringWidth(g, label) > getWidth()
                    || ScreenTools.getStringHeight(g, label) > getHeight()) {
                final Font f = new Font(getLabelFont().getFontName(),
                        getLabelFont().getStyle(), minFontSize);
                g.setFont(f);
                if (ScreenTools.getStringWidth(g, label) > getWidth()
                        || ScreenTools.getStringHeight(g, label) > getHeight()) {
                    g.setFont(getLabelFont());
                    label = "..";
                }
            }
            Point pos = ScreenTools.getStringPosition(g, label, rect.getSize());
            g.drawString(label, pos.x, pos.y);
        }
    }

}
