/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.ui.swing.components;

import java.awt.Graphics2D;
import java.util.logging.Logger;

import net.sf.jedule.ui.tools.GraphicsObject;

/**
 * @author sascha
 *
 */
public class SwingArrowObject extends GraphicsObject {

  public enum ARRAY_TYPE { HORIZONTAL, VERTICAL };

  private final ARRAY_TYPE type;
  private final int length;
  private final int x;
  private final int y;
  private final boolean filled;

  private final static Logger LOGGER = Logger.getLogger(SwingArrowObject.class
      .getName());

  public SwingArrowObject(ARRAY_TYPE type, int x, int y, int length, boolean filled) {
    this.type = type;
    this.x = x;
    this.y = y;
    this.length = length;
    this.filled = filled;
  }

  /* (non-Javadoc)
   * @see net.sf.jedule.ui.tools.GraphicsObject#draw(java.awt.Graphics2D)
   */
  @Override
  public void draw(Graphics2D g) {
    switch (type) {
    case HORIZONTAL:
      drawHorizontal(g);
      break;
    case VERTICAL:
      drawVertical(g);
      break;
    default:
      LOGGER.severe("BUG: unknown type");
      break;
    }
  }

  protected void drawVertical(Graphics2D g) {
    int width = getArrowWidth(length);

    int x1 = x + width;
    int y1 = y + length;
    int x2 = x - width;
    int y2 = y + length;

    if( filled ) {
      g.drawPolygon(new int[]{x,x1,x2}, new int[]{y,y1,y2}, 3);
      g.fillPolygon(new int[]{x,x1,x2}, new int[]{y,y1,y2}, 3);
    } else {
      g.drawLine(x1, y1, x, y);
      g.drawLine(x2, y2, x, y);
    }

  }


  protected void drawHorizontal(Graphics2D g) {

    int height =  getArrowWidth(length);

    int x1 = x - length;
    int y1 = y - height;
    int x2 = x - length;
    int y2 = y + height;

    if( filled ) {
      g.drawPolygon(new int[]{x,x1,x2}, new int[]{y,y1,y2}, 3);
      g.fillPolygon(new int[]{x,x1,x2}, new int[]{y,y1,y2}, 3);
    } else {
      g.drawLine(x1, y1, x, y);
      g.drawLine(x2, y2, x, y);
    }

  }

  protected int getArrowWidth(int length) {
    return Math.max(length/4, 2);
  }

  public int getLength() {
    return length;
  }

}
